/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2019 All Rights Reserved. 
 */
package com.abssqr.plat.common.model.domain.ast;

import com.abssqr.plat.common.facade.base.CreationAwareEntity;
import com.abssqr.plat.common.facade.enums.EventCodeEnum;
import com.abssqr.plat.common.facade.enums.LoanAstStatusEnum;
import com.abssqr.plat.common.facade.enums.OwnStatusEnum;
import com.abssqr.plat.common.facade.enums.PriceTypeEnum;
import com.abssqr.plat.common.facade.enums.ProfitTypeEnum;
import com.abssqr.plat.common.model.seq.SequenceUtil;
import com.abssqr.plat.common.model.utils.LoanAstUtil;
import com.general.system.common.model.Money;
import com.general.system.common.util.VarChecker;
import com.google.common.collect.Maps;
import org.apache.commons.lang3.StringUtils;

import java.math.BigDecimal;
import java.util.Date;
import java.util.Map;

/**
 * 机构资产
 *
 * @author zhenxuan.luo
 * @version com.abssqr.plat.common.model.domain.ast: LoanAst.java, v 0.1 2019-11-13 下午14:08 zhenxuan.luo Exp $ 
 */
public class LoanAst extends CreationAwareEntity {
    
    /**
     * loanAstNo 机构资产编号.
     */
    private String                           loanAstNo;
    /**
     * astNo 资产编号.
     */
    private String                           astNo;
    /**
     * outAstNo 外部资产编号.
     */
    private String                           outAstNo;
    /**
     * ownPack 所属资产包.
     */
    private String                           ownPack;
    /**
     * ownOrg 所属机构.
     */
    private String                           ownOrg;
    /**
     * lenderOrgCode 放款机构.
     */
    private String                           lenderOrgCode;
    /**
     * prodNo 产品编码.
     */
    private String                           prodNo;
    /**
     * outProdNo 外部产品编码.
     */
    private String                           outProdNo;
    /**
     * userId 用户ID.
     */
    private String                           userId;
    /**
     * termNo 期次编号.
     */
    private int                              termNo;
    /**
     * prinBal 本金余额.
     */
    private Money                            prinBal        = new Money();
    /**
     * paidPrinAmt 已还本金.
     */
    private Money                            paidPrinAmt    = new Money();
    /**
     * costAmt 资产成本.
     */
    private Money                            costAmt        = new Money();
    /**
     * astPrice 资产定价.
     */
    private BigDecimal                       astPrice       = BigDecimal.ZERO;
    /**
     * astPriceType 定价类型.
     */
    private PriceTypeEnum                    astPriceType;
    /**
     * inRule 资产流入规则.
     */
    private String                           inRule;
    /**
     * inDate 资产流入日期.
     */
    private Date                             inDate;
    /**
     * inAstNo 转入机构资产编号.
     */
    private String                           inAstNo;
    /**
     * astStatus 资产状态：  NOR：正常所属  CLR：已清偿（转出或还款）.
     */
    private LoanAstStatusEnum                astStatus;
    /**
     * clearEvnt 清偿事件.
     */
    private EventCodeEnum                    clearEvnt;
    /**
     * clearDate 清偿日期.
     */
    private Date                             clearDate;
    /**
     * extInf 扩展信息.
     */
    private String                           extInf;
    /**
     * lastUpdateDate 最近变更日期.
     */
    private Date                             lastUpdateDate;
    /**
     *
     */
    private LoanInstalDomain                 refInstal;
    /**
     * 所管理的机构收益：
     * <bold>注意：列表里的收益不一定全归当前资产所属机构所有，可能交易分润产生的机构收益，归属分润所属机构</bold>
     * 一条机构资产，必定关联起码一条类型为own的机构资产收益
     */
    private Map<Integer, LoanAstProfitGroup> loanAstProfits = Maps.newHashMap();
    
    public LoanAst() {
        super(true);
    }
    
    public LoanAst(boolean newModel) {
        super(newModel);
    }
    
    public OwnStatusEnum getOwnStatus() {
        if (StringUtils.equals(lenderOrgCode, ownOrg)) {
            return OwnStatusEnum.INIT;
        } else {
            return OwnStatusEnum.TRANS_OUT;
        }
    }
    
    public boolean isTransfer() {
        return StringUtils.equals(this.ownOrg, this.lenderOrgCode);
    }
    
    public void checkSelf() {
        if (!refInstal.getStatus().isActive()) {
            VarChecker.checkArgument(astStatus != LoanAstStatusEnum.NORMAL,
                    "分期[{0}-{1}]已结清,机构资产[{2}]持有状态[{3}]不能为正常",
                    refInstal.getAstNo(), refInstal.getTermNo(), this.loanAstNo, this.astStatus);
        }
        
        for (LoanAstProfitGroup loanAstProfitGroup : loanAstProfits.values()) {
            loanAstProfitGroup.checkSelf(this);
        }
        
        if (this.getAstStatus() != LoanAstStatusEnum.NORMAL) {
            VarChecker.checkZero(prinBal, "已结清机构资产[{0}]本金必须为0-[{1}]", loanAstNo, prinBal);
        }
    }
    
    public Map<Integer, LoanAstProfitGroup> getLoanAstProfits() {
        return loanAstProfits;
    }
    
    public LoanAst cloneOrTermAdj(LoanInstalDomain inInstal, Date accountDate) {
        LoanAst inAst = new LoanAst();
        inAst.setLoanAstNo(SequenceUtil.genLoanId(inInstal.getRefLoan()));
        inAst.setAstNo(this.getAstNo());
        inAst.setOutAstNo(this.getOutAstNo());
        inAst.setOwnPack(this.getOwnPack());
        inAst.setOwnOrg(this.getOwnOrg());
        inAst.setLenderOrgCode(this.getLenderOrgCode());
        inAst.setProdNo(this.getProdNo());
        inAst.setOutProdNo(this.getOutProdNo());
        inAst.setUserId(this.getUserId());
        inAst.setTermNo(inInstal.getTermNo());
        inAst.setAstStatus(LoanAstStatusEnum.NORMAL);
        inAst.setLastUpdateDate(accountDate);
        inAst.setAstPrice(this.getAstPrice());
        inAst.setAstPriceType(this.getAstPriceType());
        inAst.setInRule(this.getInRule());
        inAst.setInDate(this.getInDate());
        inAst.setInAstNo(this.getInAstNo());
    
        inAst.setRefInstal(inInstal);
        inInstal.addLoanAst(inAst);
    
        LoanAstProfit inAstProfit = LoanAstUtil.buildDefaultProfit(inInstal, inAst, accountDate);
        LoanAstProfit outAstProfit = this.getLoanAstProfit(this.getTermNo(), ProfitTypeEnum.OWN);
        inAstProfit.setSplitOrgCode(outAstProfit.getSplitOrgCode());
        inAstProfit.setSplitRate(outAstProfit.getSplitRate());

        return inAst;
    }
    
    public LoanAstProfitGroup addLoanAstProfit(LoanAstProfit loanAstProfit) {
        Integer srcTermNo = loanAstProfit.getSrcTermNo();
        
        loanAstProfit.setRefLoanAst(this);
        
        if (!loanAstProfits.containsKey(srcTermNo)) {
            loanAstProfits.put(srcTermNo, new LoanAstProfitGroup());
        }
        
        LoanAstProfitGroup profitGroup = loanAstProfits.get(srcTermNo);
        
        switch (loanAstProfit.getProfitType()) {
            case OWN:
                VarChecker.checkArgument(profitGroup.getOwnAstProfit() == null, "机构资产[{0}]的持有收益已创建[tn:{1} - stn:{2}]", this.loanAstNo, this.termNo, srcTermNo);
                profitGroup.setOwnAstProfit(loanAstProfit);
                break;
            case SPLIT:
                VarChecker.checkArgument(profitGroup.getSplitAstProfit() == null, "机构资产[{0}]的分润收益已创建[tn:{1} - stn:{2}]", this.loanAstNo, this.termNo, srcTermNo);
                profitGroup.setSplitAstProfit(loanAstProfit);
                break;
        }
        
        return profitGroup;
    }
    
    public LoanAstProfit getLoanAstProfit(int srcTermNo, ProfitTypeEnum profitType) {
        LoanAstProfitGroup loanAstProfitGroup = getLoanAstProfitGrp(srcTermNo);
        
        if (null == loanAstProfitGroup) {
            return null;
        }
        
        switch (profitType) {
            case OWN:
                return loanAstProfitGroup.getOwnAstProfit();
            case SPLIT:
                return loanAstProfitGroup.getSplitAstProfit();
        }
        return null;
    }
    
    public LoanAstProfitGroup getLoanAstProfitGrp(int srcTermNo) {
        LoanAstProfitGroup loanAstProfitGroup = loanAstProfits.get(srcTermNo);
        
        if (null == loanAstProfitGroup) {
            return null;
        }
        
        return loanAstProfitGroup;
    }
    
    public String getLoanAstNo() {
        return loanAstNo;
    }
    
    public void setLoanAstNo(String loanAstNo) {
        this.loanAstNo = loanAstNo;
    }
    
    public String getAstNo() {
        return astNo;
    }
    
    public void setAstNo(String astNo) {
        this.astNo = astNo;
    }
    
    public String getOutAstNo() {
        return outAstNo;
    }
    
    public void setOutAstNo(String outAstNo) {
        this.outAstNo = outAstNo;
    }
    
    public String getOwnPack() {
        return ownPack;
    }
    
    public void setOwnPack(String ownPack) {
        this.ownPack = ownPack;
    }
    
    public String getOwnOrg() {
        return ownOrg;
    }
    
    public void setOwnOrg(String ownOrg) {
        this.ownOrg = ownOrg;
    }
    
    public String getLenderOrgCode() {
        return lenderOrgCode;
    }
    
    public void setLenderOrgCode(String lenderOrgCode) {
        this.lenderOrgCode = lenderOrgCode;
    }
    
    public String getProdNo() {
        return prodNo;
    }
    
    public void setProdNo(String prodNo) {
        this.prodNo = prodNo;
    }
    
    public String getOutProdNo() {
        return outProdNo;
    }
    
    public void setOutProdNo(String outProdNo) {
        this.outProdNo = outProdNo;
    }
    
    public String getUserId() {
        return userId;
    }
    
    public void setUserId(String userId) {
        this.userId = userId;
    }
    
    public int getTermNo() {
        return termNo;
    }
    
    public void setTermNo(int termNo) {
        this.termNo = termNo;
    }
    
    public Money getPrinBal() {
        return prinBal;
    }
    
    public void setPrinBal(Money prinBal) {
        this.prinBal = prinBal;
    }
    
    public Money getPaidPrinAmt() {
        return paidPrinAmt;
    }
    
    public void setPaidPrinAmt(Money paidPrinAmt) {
        this.paidPrinAmt = paidPrinAmt;
    }
    
    public Money getCostAmt() {
        return costAmt;
    }
    
    public void setCostAmt(Money costAmt) {
        this.costAmt = costAmt;
    }
    
    public BigDecimal getAstPrice() {
        return astPrice;
    }
    
    public void setAstPrice(BigDecimal astPrice) {
        this.astPrice = astPrice;
    }
    
    public PriceTypeEnum getAstPriceType() {
        return astPriceType;
    }
    
    public void setAstPriceType(PriceTypeEnum astPriceType) {
        this.astPriceType = astPriceType;
    }
    
    public String getInRule() {
        return inRule;
    }
    
    public void setInRule(String inRule) {
        this.inRule = inRule;
    }
    
    public Date getInDate() {
        return inDate;
    }
    
    public void setInDate(Date inDate) {
        this.inDate = inDate;
    }
    
    public String getInAstNo() {
        return inAstNo;
    }
    
    public void setInAstNo(String inAstNo) {
        this.inAstNo = inAstNo;
    }
    
    public LoanAstStatusEnum getAstStatus() {
        return astStatus;
    }
    
    public void setAstStatus(LoanAstStatusEnum astStatus) {
        this.astStatus = astStatus;
    }
    
    public EventCodeEnum getClearEvnt() {
        return clearEvnt;
    }
    
    public void setClearEvnt(EventCodeEnum clearEvnt) {
        this.clearEvnt = clearEvnt;
    }
    
    public Date getClearDate() {
        return clearDate;
    }
    
    public void setClearDate(Date clearDate) {
        this.clearDate = clearDate;
    }
    
    public String getExtInf() {
        return extInf;
    }
    
    public void setExtInf(String extInf) {
        this.extInf = extInf;
    }
    
    public Date getLastUpdateDate() {
        return lastUpdateDate;
    }
    
    public void setLastUpdateDate(Date lastUpdateDate) {
        this.lastUpdateDate = lastUpdateDate;
    }
    
    public LoanInstalDomain getRefInstal() {
        return refInstal;
    }
    
    public void setRefInstal(LoanInstalDomain refInstal) {
        this.refInstal = refInstal;
    }
}